@@ -63,9 +63,12 @@ banner_free(banner_t *banner)
 {
   if (banner)
   {
-    free(banner->template_file);
-    free(banner->header);
-    free(banner->footer);
+    if (banner->template_file)
+      free(banner->template_file);
+    if (banner->header)
+      free(banner->header);
+    if (banner->footer)
+      free(banner->footer);
     free(banner);
   }
 }
@@ -224,7 +227,12 @@ banner_new_from_file(const char *filename,
       break;
   }
 
-  banner = calloc(1, sizeof *banner);
+  if ((banner = calloc(1, sizeof *banner)) == NULL)
+  {
+    if (log)
+      log(ld, CF_LOGLEVEL_ERROR, "cfFilterBannerToPDF: Not enough memory");
+    goto out;
+  }
 
   while (getline(&line, &len, f) != -1)
   {
@@ -248,12 +256,12 @@ banner_new_from_file(const char *filename,
       while (*key == '%')
 	key ++;
 
-    if (!strcasecmp(key, "template"))
+    if (!banner->template_file && !strcasecmp(key, "template"))
       banner->template_file = template_path(value, datadir);
-    else if (!strcasecmp(key, "header"))
-      banner->header = strdup(value);
-    else if (!strcasecmp(key, "footer"))
+    else if (!banner->header && !strcasecmp(key, "header"))
       banner->header = strdup(value);
+    else if (!banner->footer && !strcasecmp(key, "footer"))
+      banner->footer = strdup(value);
     else if (!strcasecmp(key, "font"))
       *num_options = cupsAddOption("banner-font",
 				   strdup(value), *num_options, options);
@@ -1003,6 +1011,8 @@ cfFilterBannerToPDF(int inputfd,         // I - File descriptor input stream
   banner_free(banner);
   if (options) cupsFreeOptions(num_options, options);
   unlink(tempfile);
+  if (inputfp)
+    fclose(inputfp);
   
   return (ret);
 }
