@@ -699,6 +699,13 @@ gs_spawn (const char *filename,
   if (log) log(ld, CF_LOGLEVEL_DEBUG,
 	       "cfFilterGhostscript: Input data feed completed");
 
+  //
+  // Initialize to exit status 'Success' first - any error exit status
+  // from children process will rewrite it, and wait for children...
+  //
+
+  status = 0;
+
   while (gspid > 0 || errpid > 0)
   {
     if ((pid = wait(&wstatus)) < 0)
@@ -744,7 +751,6 @@ gs_spawn (const char *filename,
       if (log) log(ld, CF_LOGLEVEL_DEBUG,
 		   "cfFilterGhostscript: %s (PID %d) exited with no errors.",
 		   (pid == gspid ? "Ghostscript" : "Logging"), pid);
-      status = 0;
     }
     if (pid == gspid)
       gspid = -1;
@@ -774,7 +780,7 @@ cfFilterGhostscript(int inputfd,            // I - File descriptor input
 {
   cf_filter_out_format_t outformat;
   char buf[BUFSIZ];
-  char *filename;
+  char *filename = NULL;
   char *icc_profile = NULL;
   char *tmp;
   char tmpstr[1024],
